package zfsearch;

use POSIX;
use integer;

$zfallowscript=1;

#
#the only one path that should be hard-coded (feel free to change it if you need it)
#
$cfgFILE="/etc/zfsearch/client.conf";

#
#default paths that will be used if you have no data in your config file (see above)
#

$cfgDATADIR="/var/spool/zfsearch";
$cfgSKELDIR="/usr/lib/zfsearch/client/skel";
$cfgHTTPPREFIX="/";

sub read_config() {
    open(C,$cfgFILE);
    while (<C>) { chomp;
      if (/^(.+)#/) {$_=$1}
      if (/^\s*DATADIR\s*=\s*(.*?)\s*$/) { $cfgDATADIR=$1}
      if (/^\s*SKELDIR\s*=\s*(.*?)\s*$/) { $cfgSKELDIR=$1}
      if (/^\s*HTTPPREFIX\s*=\s*(.*?)\s*$/) { $cfgHTTPPREFIX=$1}
    }
    close(C);
}
sub DATADIR() {return $cfgDATADIR}
sub SKELDIR() {return $cfgSKELDIR}
sub HTTPPREFIX() {return $cfgHTTPPREFIX}

sub log($) {
    my $params=$_[1]; # hash for requested values
    print STDERR "==============\n";
    foreach $k (keys %$params) {
     print STDERR "$k = $$params{$k}\n";
    }
}

sub parse_params($$) {
    my $req=$_[1]; # request CGI
    my $params=$_[2]; # hash for requested values
    
    my @pl=$req->param;
    foreach (@pl) {$$params{$_}=$req->param($_)}
}

sub setup_gettext($) {
    my $langlist=$_[1]; # list of languages
    
    # Analyse accept-language here
    foreach (split(/,/, $langlist)) {
        /(\w+)/;
        open (T,$cfgSKELDIR."/translate.$1") || next;
        while (<T>) {chomp;
            if (/\s*(.+?)\s*=\s*"(.+)"\s*/) {$trans{$1}=$2}
        }
        close(T);
        last;
    }
}

sub gettext($) {
    return $trans{$_[1]}
}

sub put_skel_mode($$) {
    my $dir=$_[1];   # dir of skel files
    my $v=$_[2];     # hash with parameters
    SWITCH:
    {
        if ($$v{'mode'} eq "R") {internal_put_skel("$dir/mode_regex.skel.html",$v);last SWITCH;}
        if ($$v{'mode'} eq "E") {internal_put_skel("$dir/mode_easy.skel.html",$v); last SWITCH;}
        if ($$v{'mode'} eq "S") {internal_put_skel("$dir/mode_smart.skel.html",$v);last SWITCH;}
        internal_put_skel("$dir/mode_smart.skel.html",$v);
    }
}

sub put_skel($$) {
    internal_put_skel($_[1],$_[2]);
}

sub internal_put_skel($$) {
    my $filename=$_[0];   # filename of skel
    my $v=$_[1];          # hash with parameters
    
    if (!open(F_SKEL, $filename)) {
        print "<H1>Error : skeleton file $filename not found</H1>";
        print '</body></html>'."\n";
        die($filename . ": no such file")
    }
    my $disp=1;
    while(<F_SKEL>){
        if ($zfallowscript != 1) {
            if (/<\/script>/) {$disp=1;next;}
            if (/<script>/) {$disp=0}
            if ($disp==0) {next}
            s/onload="\w+\(\)"//gi;
            s/onclick="\w+\(\)"//gi;
            s/onchange="\w+\(\)"//gi;
        }
        if ( /value="(\w)".+\(\%([\w|-]+)\%\)/ ) { #controls with value
            my $control=$2,$value=$1,$translated;
            if ($$v{$control} eq $value) {$translated = 'checked'} else {$translated=''}
            s/\(\%$control\%\)/$translated/g;
        } else {
            if ( /\(\%([\w|-]+)\%\)/ ) { #controls with any value
                my $control=$1,$translated;
                $translated=$$v{$control};
                s/\(\%$control\%\)/$translated/g;
            }
        }
        if ( /\[\%([\w|-]+)\%\]/ ) { # gettext
            my $word=$1;
            my $translated = $trans{$1};
            s/\[\%$word\%\]/$translated/;
        }
        if ( /\{\%HTTPPREFIX\%\}/ ) { # config
            s/\{\%HTTPPREFIX\%\}/$cfgHTTPPREFIX/g;
        }
        print "$_\n";
    }
    close (F_SKEL);
}

sub put_top($) {
    my $v=$_[1];          # hash with parameters
    my $dis=$_[2];        # disable java script when =1

    print "Content-Type: text/html; charset=utf-8\nExpires: -1\n\n";

    if ($dis==1) {
        my $oldza=$zfallowscript;
        $zfallowscript=0;
    }
    internal_put_skel($cfgSKELDIR."/index_top.skel.html",$v);
    SWITCH:
    {
        if ($$v{'mode'} eq "R") {internal_put_skel($cfgSKELDIR."/mode_regex_nav.skel.html",$v);last SWITCH;}
        if ($$v{'mode'} eq "E") {internal_put_skel($cfgSKELDIR."/mode_easy_nav.skel.html",$v); last SWITCH;}
        if ($$v{'mode'} eq "S") {internal_put_skel($cfgSKELDIR."/mode_smart_nav.skel.html",$v);last SWITCH;}
        if ($$v{'mode'} eq "T") {internal_put_skel($cfgSKELDIR."/stat_nav.skel.html",$v);last SWITCH;}
        internal_put_skel($cfgSKELDIR."/mode_smart_nav.skel.html",$v);
    }
    if ($dis==1) {
        $zfallowscript=$oldza;
    }
}

sub put_bottom($) {
    my $v=$_[1];          # hash with parameters
    internal_put_skel($cfgSKELDIR."/index_bottom.skel.html",$v);
}

sub put_paging($$$$$) {
    my $cf=$_[1];
    my $cd=$_[2];
    my $portion_size=$_[3];
    my $pcurr=$_[4];
    my $query=$_[5];
    my $p,$pmax;
    my $cmax;
    
    if ($cf>$cd) {$cmax=$cf} else {$cmax=$cd}
    
    if ($cmax>$portion_size) {
        print "<center><table border=0>\n\t<tr><td align=right>\n\t";
        
        if ($pcurr>0) { #prev
            $p = $pcurr - 1;
            $$query{"portion"}=$p;
            print "\t<a href=\"/cgi-bin/search_names.pl?".zfsearch->query2str($query)."\">", $trans{"prev"}, "</a>\n";
        } else {
            print "\t", $trans{"prev"}, "\n";
        }
        
        $p=$pcurr-5;
        if ($p<0) {$p=0;}
        $pmax=int($cmax/$portion_size);
        if ($pmax>($p+19)) {$pmax=$p+19}
        for (;$p<=$pmax; $p++) {
            if ($p==$pcurr) {
                print "\t</td><td align=center>\n\t".($p+1)."\n";
            } else {
            $$query{"portion"}=$p;
                print "\t</td><td align=center>\n\t<a href=\"/cgi-bin/search_names.pl?".zfsearch->query2str($query)."\">".($p+1)."</a>\n";
            }
        }
        print "\t</td><td align=left>\n\t";
    
        if ($pcurr<$pmax) { #next
            $p = $pcurr + 1;
            $$query{"portion"}=$p;
            print "\t<a href=\"/cgi-bin/search_names.pl?".zfsearch->query2str($query)."\">", $trans{"next"}, "</a>\n";
        } else {
            print "\t", $trans{"next"}, "\n";
        }
        print "\t</td></tr></table></center>\n";
    }
}

sub mmeta($) {
    my $r=$_[0];
    $r=~s/;/%3B/g;
    $r=~s/&/%26/g;
    $r=~s/\+/%2B/g;
    $r=~s/=/%3D/g;
    $r=~s/ /%20/g;
    return $r;
}

sub query2str($) {
    my $s="";
    $params=$_[1];
    foreach (keys %$params) {
        $s.=";$_=".mmeta($$params{$_});
    }
    return substr($s,1);
}

1;
